@@ -35,30 +35,30 @@
 #include <setjmp.h>
 #include <signal.h>
 #ifndef RETSIGTYPE
-#define RETSIGTYPE void
+# define RETSIGTYPE void
 #endif
 
-#ifdef HAVE_STRING_H
-#include <string.h>
+#if HAVE_STRING_H
+# include <string.h>
 #else
-#include <strings.h>
-#define strchr index
-#define strrchr rindex
+# include <strings.h>
+# define strchr index
+# define strrchr rindex
 #endif
 
-#ifdef MSDOS
-#define MSDOS_or_OS2 1
-#include <dir.h>
-#define unlink dummy1
-#include <io.h>
-#undef unlink
-#include <fcntl.h>
+#if MSDOS
+# define MSDOS_or_OS2 1
+# include <dir.h>
+# define unlink dummy1
+# include <io.h>
+# undef unlink
+# include <fcntl.h>
 #endif
 
-#ifdef OS2
-#define MSDOS_or_OS2 1
-#include <io.h>
-#include <fcntl.h>
+#if OS2
+# define MSDOS_or_OS2 1
+# include <io.h>
+# include <fcntl.h>
 #endif
 
 #include <errno.h>
@@ -85,7 +85,7 @@ extern int errno;
    
    Noah Friedman <friedman@gnu.ai.mit.edu> suggests opening the file with
    O_EXCL, and when the open presumably fails, call tmpnam again, or try
-   the mktemp routine in the GNU C library...maybe that will work better.
+   the mktemp routine in the GNU C library: maybe that will work better.
    
    Michael I Bushnell <mib@gnu.ai.mit.edu> suggests always using tmpfile,
    which opens the file too, using the O_EXCL option to open.
@@ -98,61 +98,38 @@ extern int errno;
    tell me if this creates a problem somewhere!  */
 
 /* The previous round used tmpnam(3).  This one tries tmpfile(3).  */
-/* #define USE_TMPNAM 1 */
+#undef USE_TMPNAM
 #define USE_TMPFILE 1
 
-#ifdef USE_TMPNAM
 /* Guarantee some value for L_tmpnam.  */
-#ifdef MSDOS
-
-#define L_tmpnam 13
-
-#else /* not MSDOS */
-
+#if USE_TMPNAM
+# if MSDOS
+#  define L_tmpnam 13
+# else
 char *tmpnam ();
-
-#ifndef L_tmpnam
-#include "pathmax.h"
-#define L_tmpnam PATH_MAX
+#  ifndef L_tmpnam
+#   include "pathmax.h"
+#   define L_tmpnam PATH_MAX
+#  endif
+# endif
 #endif
 
-#endif /* not MSDOS */
-#endif /* USE_TMPNAM */
-
-#ifdef USE_TMPFILE
-
-FILE *tmpfile _((void));
-
-#endif /* USE_TMPFILE */
-
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
+#if USE_TMPFILE
+FILE *tmpfile __P ((void));
 #endif
 
-#ifdef MSDOS
+#if WITH_DMALLOC
+# include <dmalloc.h>
+#endif
+
+#if MSDOS
 /* Increase stack size, so init_recode_rfc1345 works.  */
 extern unsigned _stklen = 10000U;
 #endif
 
 /* Variables.  */
 
-const char *const copyright_string = "\
-This program is free software; you can redistribute it and/or modify\n\
-it under the terms of the GNU General Public License as published by\n\
-the Free Software Foundation; either version 2, or (at your option)\n\
-any later version.\n\
-\n\
-This program is distributed in the hope that it will be useful,\n\
-but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
-GNU General Public License for more details.\n\
-\n\
-You should have received a copy of the GNU General Public License\n\
-along with this program; if not, write to the Free Software\n\
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n";
-
 /* The name this program was run with. */
-const char *program_path;
 const char *program_name;
 
 /* If non-zero, display usage information and exit.  */
@@ -260,7 +237,42 @@ CHARSET *rfc1345;		/* special RFC 1345 charset value */
 const STEP *sequence[MAX_SEQUENCE];
 int length_of_sequence;
 
-/* Signal handling.  */
+/* Error handling.  */
+
+/*---------------------------------------------------------------------.
+| Print the source location of error when PERROR or PERROR1 are used.  |
+`---------------------------------------------------------------------*/
+
+static const char *error_file_reference = NULL;
+static int error_line_reference = 0;
+
+static void
+error_print_prefix (void)
+{
+  fflush (stdout);
+  if (error_file_reference)
+    fprintf (stderr, "%s:%d: ", error_file_reference, error_line_reference);
+  else
+    fprintf (stderr, "%s: ", program_name);
+}
+
+#define PERROR(Message) \
+  error ((error_file_reference = __FILE__,				\
+	  error_line_reference = __LINE__,				\
+	  EXIT_FAILURE),						\
+	 errno, "%s", (Message))
+
+#define PERROR1(Format, Argument) \
+  error ((error_file_reference = __FILE__,				\
+	  error_line_reference = __LINE__,				\
+	  EXIT_FAILURE),						\
+	 errno, (Format), (Argument))
+
+#define PERROR2(Format, Argument1, Argument2) \
+  error ((error_file_reference = __FILE__,				\
+	  error_line_reference = __LINE__,				\
+	  EXIT_FAILURE),						\
+	 errno, (Format), (Argument1), (Argument2))
 
 /*-----------------.
 | Signal handler.  |
@@ -283,7 +295,9 @@ signal_handler (int number)
 static void
 setup_signals (void)
 {
+#ifdef SIGPIPE
   signal (SIGPIPE, signal_handler);
+#endif
 #if 0
   signal (SIGINT, signal_handler);
   signal (SIGTERM, signal_handler);
@@ -305,19 +319,19 @@ quality_to_string (QUALITY quality)
       abort ();
 
     case REVERSIBLE:
-      return "reversible";
+      return _("reversible");
 
     case ONE_TO_ONE:
-      return "one to one";
+      return _("one to one");
 
     case MANY_TO_ONE:
-      return "many to one";
+      return _("many to one");
 
     case ONE_TO_MANY:
-      return "one to many";
+      return _("one to many");
 
     case MANY_TO_MANY:
-      return "many to many";
+      return _("many to many");
     }
 }
 
@@ -445,7 +459,7 @@ new_single_step (void)
   STEP *step;
 
   if (number_of_single_steps == MAX_SINGLE_STEPS)
-    error (EXIT_FAILURE, 0, "MAX_SINGLE_STEPS is too small");
+    error (EXIT_FAILURE, 0, _("MAX_SINGLE_STEPS is too small"));
 
   step = single_step_array + number_of_single_steps++;
   step->init_recode = NULL;
@@ -507,7 +521,7 @@ invert_table (const unsigned char *table)
     {
       if (flag[table[counter]])
 	{
-	  error (0, 0, "Codes %3d and %3d both recode to %3d",
+	  error (0, 0, _("Codes %3d and %3d both recode to %3d"),
 		 result[table[counter]], counter, table[counter]);
 	  table_error = 1;
 	}
@@ -521,8 +535,8 @@ invert_table (const unsigned char *table)
     {
       for (counter = 0; counter < 256; counter++)
 	if (!flag[counter])
-	  error (0, 0, "No character recodes to %3d", counter);
-      error (EXIT_FAILURE, 0, "Cannot invert given one-to-one table");
+	  error (0, 0, _("No character recodes to %3d"), counter);
+      error (EXIT_FAILURE, 0, _("Cannot invert given one-to-one table"));
     }
   return result;
 }
@@ -575,22 +589,24 @@ complete_pairs (STEP *step, int first_half_implied,
 	{
 	  if (!table_error)
 	    {
-	      error (0, 0, "Following diagnostics for `%s' to `%s'",
+	      error (0, 0, _("Following diagnostics for `%s' to `%s'"),
 		     step->before->name, step->after->name);
 	      table_error = 1;
 	    }
-	  error (0, 0, "Pair no. %d: { %3d, %3d } conflicts with { %3d, %3d }",
+	  error (0, 0,
+		 _("Pair no. %d: { %3d, %3d } conflicts with { %3d, %3d }"),
 		 counter, left, right, left, left_table[left]);
 	}
       else if (right_flag[right])
 	{
 	  if (!table_error)
 	    {
-	      error (0, 0, "Following diagnostics for `%s' to `%s'",
+	      error (0, 0, _("Following diagnostics for `%s' to `%s'"),
 		     step->before->name, step->after->name);
 	      table_error = 1;
 	    }
-	  error (0, 0, "Pair no. %d: { %3d, %3d } conflicts with { %3d, %3d }",
+	  error (0, 0,
+		 _("Pair no. %d: { %3d, %3d } conflicts with { %3d, %3d }"),
 		 counter, left, right, right_table[right], right);
 	}
       else
@@ -668,7 +684,7 @@ complete_pairs (STEP *step, int first_half_implied,
 
       if (table_error)
 	error (EXIT_FAILURE, 0,
-	       "Cannot complete table from set of known pairs");
+	       _("Cannot complete table from set of known pairs"));
 
       /* Close the table with small permutation cycles.  */
 
@@ -786,31 +802,31 @@ output_header_file (void)
 
   /* Print the header of the header file.  */
 
-  printf ("/* Conversion table from `%s' charset to `%s' charset.\n",
+  printf (_("/* Conversion table from `%s' charset to `%s' charset.\n"),
 	  before_full_name, after_full_name);
-  printf ("   Generated mechanically by GNU %s %s.\n", PRODUCT, VERSION);
+  printf (_("   Generated mechanically by GNU %s %s.\n"), PACKAGE, VERSION);
   printf ("\n");
   switch (sequence[0]->quality)
     {
     case REVERSIBLE:
-      printf ("   The recoding should be reversible.\n");
+      printf (_("   The recoding should be reversible.\n"));
       break;
 
     case ONE_TO_ONE:
-      printf ("   The recoding might not be reversible.\n");
+      printf (_("   The recoding might not be reversible.\n"));
       break;
 
     case MANY_TO_ONE:
-      printf ("   Programming is needed to handle multichar input.\n");
+      printf (_("   Programming is needed to handle multichar input.\n"));
       break;
 
     case ONE_TO_MANY:
-      printf ("   Each input char transforms into an output string.\n");
+      printf (_("   Each input char transforms into an output string.\n"));
       break;
 
     case MANY_TO_MANY:
-      printf ("   Each input char transforms into an output string,\n");
-      printf ("   programming is needed to handle multichar input.\n");
+      printf (_("   Each input char transforms into an output string,\n"));
+      printf (_("   programming is needed to handle multichar input.\n"));
       break;
     }
   printf ("*/\n");
@@ -939,7 +955,7 @@ output_header_file (void)
     }
 
   else
-    error (EXIT_FAILURE, 0, "No table to print");
+    error (EXIT_FAILURE, 0, _("No table to print"));
 
   free (name);
 }
@@ -1147,9 +1163,9 @@ echo_sequence (void)
   int counter;			/* index into sequence */
 
   if (length_of_sequence < 0)
-    fprintf (stderr, "UNACHIEVABLE recoding!\n");
+    fprintf (stderr, _("UNACHIEVABLE recoding!\n"));
   else if (length_of_sequence == 0)
-    fprintf (stderr, "Mere copy for the trivial recoding\n");
+    fprintf (stderr, _("Mere copy for the trivial recoding\n"));
   else
     {
       quality = REVERSIBLE;
@@ -1252,7 +1268,7 @@ find_sequence (CHARSET *before, CHARSET *after)
 	      || step->one_to_one != one_to_same)
 	    {
 	      if (length_of_sequence == MAX_SEQUENCE)
-		error (EXIT_FAILURE, 0, "MAX_SEQUENCE is too small");
+		error (EXIT_FAILURE, 0, _("MAX_SEQUENCE is too small"));
 	      sequence[length_of_sequence++] = step;
 	      if (step->init_recode)
 		{
@@ -1344,7 +1360,8 @@ simplify_sequence (void)
 	sequence[out++] = step;
       }
     else if (sequence[in]->before == rfc1345)
-      error (EXIT_FAILURE, 0, "You may not explicitely recode from RFC 1345");
+      error (EXIT_FAILURE, 0,
+	     _("You may not explicitely recode from RFC 1345"));
     else if (sequence[in]->after == rfc1345)
       {
 
@@ -1461,7 +1478,7 @@ simplify_sequence (void)
 
   if (saved_steps > 0 && verbose_option)
     {
-      fprintf (stderr, "Simplified to: ");
+      fprintf (stderr, _("Simplified to: "));
       echo_sequence ();
     }
 }
@@ -1546,7 +1563,7 @@ execute_pass_sequence (const char *input_name, const char *output_name)
   FILE *input_file;		/* input file to recoding step */
   FILE *output_file;		/* output file from recoding step */
   int reversible;		/* reversibility of recoding */
-#ifdef USE_TMPNAM
+#if USE_TMPNAM
   char *temp_input_name;	/* step input file name */
   char *temp_output_name;	/* step output file name */
   char temp_name_1[L_tmpnam];	/* one temporary file name */
@@ -1554,11 +1571,11 @@ execute_pass_sequence (const char *input_name, const char *output_name)
   char *exchange_temp;		/* for exchanging temporary names */
 #endif
 
-#ifdef USE_TMPNAM
+#if USE_TMPNAM
 
   /* Choose names for intermediate files.  Use "" for delaying them.  */
 
-#ifdef MSDOS_or_OS2
+#if MSDOS_or_OS2
   strcpy (temp_name_1, "recodex1.tmp");
   strcpy (temp_name_2, "recodex2.tmp");
 #else
@@ -1585,18 +1602,16 @@ execute_pass_sequence (const char *input_name, const char *output_name)
 	  if (!input_name)
 	    input_file = stdin;
 	  else if (input_file = fopen (input_name, "r"), input_file == NULL)
-	    error (EXIT_FAILURE, errno, "%s:%d: fopen (%s)",
-		   __FILE__, __LINE__, input_name);
+	    PERROR1 ("fopen (%s)", input_name);
 	}
       else
 	{
-#ifdef USE_TMPNAM
+#if USE_TMPNAM
 	  if (input_file = fopen (temp_input_name, "r"), input_file == NULL)
-	    error (EXIT_FAILURE, errno, "%s:%d: fopen (%s)",
-		   __FILE__, __LINE__, temp_input_name);
+	    PERROR1 ("fopen (%s)", temp_input_name);
 #endif
 
-#ifdef USE_TMPFILE
+#if USE_TMPFILE
 	  rewind (input_file);
 #endif
 	}
@@ -1608,25 +1623,22 @@ execute_pass_sequence (const char *input_name, const char *output_name)
 	  if (!output_name)
 	    output_file = stdout;
 	  else if (output_file = fopen (output_name, "w"), output_file == NULL)
-	    error (EXIT_FAILURE, errno, "%s:%d: fopen (%s)",
-		   __FILE__, __LINE__, output_name);
+	    PERROR1 ("fopen (%s)", output_name);
 	}
       else
 	{
-#ifdef USE_TMPNAM
-#ifdef MSDOS_or_OS2
+#if USE_TMPNAM
+# if MSDOS_or_OS2
 	  if (*temp_output_name == '\0')
 	    tmpnam (temp_output_name);
-#endif
+# endif
 	  if (output_file = fopen (temp_output_name, "w"), output_file == NULL)
-	    error (EXIT_FAILURE, errno, "%s:%d: fopen (%s)",
-		   __FILE__, __LINE__, temp_output_name);
+	    PERROR1 ("fopen (%s)", temp_output_name);
 #endif
 
-#ifdef USE_TMPFILE
+#if USE_TMPFILE
 	  if (output_file = tmpfile (), output_file == NULL)
-	    error (EXIT_FAILURE, errno, "%s:%d: tmpfile ()",
-		   __FILE__, __LINE__);
+	    PERROR ("tmpfile ()");
 #endif
 	}
 
@@ -1646,7 +1658,7 @@ execute_pass_sequence (const char *input_name, const char *output_name)
       else
 	{
 	  fclose (input_file);
-#ifdef USE_TMPNAM
+#if USE_TMPNAM
 	  unlink (temp_input_name);
 #endif
 	}
@@ -1660,7 +1672,7 @@ execute_pass_sequence (const char *input_name, const char *output_name)
 	}
       else
 	{
-#ifdef USE_TMPNAM
+#if USE_TMPNAM
 	  fclose (output_file);
 
 	  exchange_temp = temp_input_name;
@@ -1668,7 +1680,7 @@ execute_pass_sequence (const char *input_name, const char *output_name)
 	  temp_output_name = exchange_temp;
 #endif
 
-#ifdef USE_TMPFILE
+#if USE_TMPFILE
 	  input_file = output_file;
 #endif
 	}
@@ -1683,7 +1695,7 @@ execute_pass_sequence (const char *input_name, const char *output_name)
 | non-reversible.							   |
 `-------------------------------------------------------------------------*/
 
-#ifdef HAVE_POPEN
+#if HAVE_POPEN
 
 static int
 execute_popen_sequence (const char *input_name, const char *output_name)
@@ -1696,7 +1708,7 @@ execute_popen_sequence (const char *input_name, const char *output_name)
 
   /* Construct a `recode' command for all recoding steps but the first.  */
 
-  strcpy (popen_command, program_path);
+  strcpy (popen_command, program_name);
   if (diaeresis_char)
     strcat (popen_command, " -c");
   if (diacritics_only)
@@ -1719,12 +1731,10 @@ execute_popen_sequence (const char *input_name, const char *output_name)
   if (!input_name)
     input_file = stdin;
   else if (input_file = fopen (input_name, "r"), input_file == NULL)
-    error (EXIT_FAILURE, errno, "%s:%d: fopen (%s)",
-	   __FILE__, __LINE__, input_name);
+    PERROR1 ("fopen (%s)", input_name);
 
   if (output_file = popen (popen_command, "w"), output_file == NULL)
-    error (EXIT_FAILURE, errno, "%s:%d: popen (%s)",
-	   __FILE__, __LINE__, popen_command);
+    PERROR1 ("popen (%s)", popen_command);
 
   step = sequence[0];
   reversible = (*step->file_recode) (step, input_file, output_file);
@@ -1742,9 +1752,9 @@ execute_popen_sequence (const char *input_name, const char *output_name)
 
 #endif /* HAVE_POPEN */
 
-#ifdef HAVE_PIPE
+#if HAVE_PIPE
 
-#ifndef HAVE_DUP2
+#if !HAVE_DUP2
 
 /*------------------------------------------------------------------------.
 | Duplicate the OLD_FD file descriptor into NEW_FD, closing NEW_FD first  |
@@ -1832,8 +1842,7 @@ execute_pipe_sequence (const char *input_name, const char *output_name)
   if (!output_name)
     output_file = stdout;
   else if (output_file = fopen (output_name, "w"), output_file == NULL)
-    error (EXIT_FAILURE, errno, "%s:%d: fopen (%s)",
-	   __FILE__, __LINE__, output_name);
+    PERROR1 ("fopen (%s)", output_name);
 
   /* Create all subprocesses, from the last to the first, and
      interconnect them.  */
@@ -1844,11 +1853,9 @@ execute_pipe_sequence (const char *input_name, const char *output_name)
        sequence_index--)
     {
       if (pipe (pipe_pair) < 0)
-	error (EXIT_FAILURE, errno, "%s:%d: pipe ()",
-	       __FILE__, __LINE__);
+	PERROR ("pipe ()");
       if (child_process = fork (), child_process < 0)
-	error (EXIT_FAILURE, errno, "%s:%d: fork ()",
-	       __FILE__, __LINE__);
+	PERROR ("fork ()");
       if (child_process == 0)
 	{
 
@@ -1856,11 +1863,9 @@ execute_pipe_sequence (const char *input_name, const char *output_name)
              and writing to the current output file; then it exits.  */
 
 	  if (close (pipe_pair[1]) < 0)
-	    error (EXIT_FAILURE, errno, "%s:%d: close ()",
-		   __FILE__, __LINE__);
+	    PERROR ("close ()");
 	  if (input_file = fdopen (pipe_pair[0], "r"), input_file == NULL)
-	    error (EXIT_FAILURE, errno, "%s:%d: fdopen ()",
-		   __FILE__, __LINE__);
+	    PERROR ("fdopen ()");
 
 	  step = sequence[sequence_index];
 	  if (!(*step->file_recode) (step, input_file, output_file))
@@ -1878,14 +1883,11 @@ execute_pipe_sequence (const char *input_name, const char *output_name)
           /* The parent redirects the current output file to the pipe.  */
 
 	  if (dup2 (pipe_pair[1], fileno (output_file)) < 0)
-	    error (EXIT_FAILURE, errno, "%s:%d: dup2 ()",
-		   __FILE__, __LINE__);
+	    PERROR ("dup2 ()");
 	  if (close (pipe_pair[0]) < 0)
-	    error (EXIT_FAILURE, errno, "%s:%d: close ()",
-		   __FILE__, __LINE__);
+	    PERROR ("close ()");
 	  if (close (pipe_pair[1]) < 0)
-	    error (EXIT_FAILURE, errno, "%s:%d: close ()",
-		   __FILE__, __LINE__);
+	    PERROR ("close ()");
 	}
     }
 
@@ -1895,8 +1897,7 @@ execute_pipe_sequence (const char *input_name, const char *output_name)
   if (!input_name)
     input_file = stdin;
   else if (input_file = fopen (input_name, "r"), input_file == NULL)
-    error (EXIT_FAILURE, errno, "%s:%d: fopen (%s)",
-	   __FILE__, __LINE__, input_name);
+    PERROR1 ("fopen (%s)", input_name);
 
   step = sequence[0];
   if (!(*step->file_recode) (step, input_file, output_file))
@@ -1918,7 +1919,7 @@ execute_pipe_sequence (const char *input_name, const char *output_name)
 
       if ((wait_status & ~(~0 << 8)) != 0
 	  && (wait_status & ~(~0 << 8)) != SIGPIPE)
-	error (0, 0, "Child process wait status is 0x%0.2x",
+	error (0, 0, _("Child process wait status is 0x%0.2x"),
 	       wait_status);
 
       /* Check for a non-zero exit from the terminating child.  */
@@ -1953,14 +1954,12 @@ execute_pipe_sequence (const char *input_name, const char *output_name)
   if (!input_name)
     input_file = stdin;
   else if (input_file = fopen (input_name, "r"), input_file == NULL)
-    error (EXIT_FAILURE, errno, "%s:%d: fopen (%s)",
-	   __FILE__, __LINE__, input_name);
+    PERROR1 ("fopen (%s)", input_name);
 
   if (!output_name)
     output_file = stdout;
   else if (output_file = fopen (output_name, "w"), output_file == NULL)
-    error (EXIT_FAILURE, errno, "%s:%d: fopen (%s)",
-	   __FILE__, __LINE__, output_name);
+    PERROR1 ("fopen (%s)", output_name);
 
   /* Create all subprocesses, from the first to the last, and
      interconnect them.  */
@@ -1971,22 +1970,18 @@ execute_pipe_sequence (const char *input_name, const char *output_name)
        sequence_index++)
     {
       if (pipe (pipe_pair) < 0)
-	error (EXIT_FAILURE, errno, "%s:%d: pipe ()",
-	       __FILE__, __LINE__);
+	PERROR ("pipe ()");
       if (child_process = fork (), child_process < 0)
-	error (EXIT_FAILURE, errno, "%s:%d: fork ()",
-	       __FILE__, __LINE__);
+	PERROR ("fork ()");
       if (child_process == 0)
 	{
           /* The child executes its recoding step, reading from the pipe
              and writing to the current output file; then it exits.  */
 
 	  if (close (pipe_pair[1]) < 0)
-	    error (EXIT_FAILURE, errno, "%s:%d: close ()",
-		   __FILE__, __LINE__);
+	    PERROR ("close ()");
 	  if (input_file = fdopen (pipe_pair[0], "r"), input_file == NULL)
-	    error (EXIT_FAILURE, errno, "%s:%d: fdopen ()",
-		   __FILE__, __LINE__);
+	    PERROR ("fdopen ()");
 
 	  step = sequence[sequence_index];
 	  if (!(*step->file_recode) (step, input_file, output_file))
@@ -2004,14 +1999,11 @@ execute_pipe_sequence (const char *input_name, const char *output_name)
           /* The parent redirects the current output file to the pipe.  */
 
 	  if (dup2 (pipe_pair[1], fileno (output_file)) < 0)
-	    error (EXIT_FAILURE, errno, "%s:%d: dup2 ()",
-		   __FILE__, __LINE__);
+	    PERROR ("dup2 ()");
 	  if (close (pipe_pair[0]) < 0)
-	    error (EXIT_FAILURE, errno, "%s:%d: close ()",
-		   __FILE__, __LINE__);
+	    PERROR ("close ()");
 	  if (close (pipe_pair[1]) < 0)
-	    error (EXIT_FAILURE, errno, "%s:%d: close ()",
-		   __FILE__, __LINE__);
+	    PERROR ("close ()");
 	}
       else
 	{
@@ -2028,8 +2020,7 @@ execute_pipe_sequence (const char *input_name, const char *output_name)
   if (!input_name)
     input_file = stdin;
   else if (input_file = fopen (input_name, "r"), input_file == NULL)
-    error (EXIT_FAILURE, errno, "%s:%d: fopen (%s)",
-	   __FILE__, __LINE__, input_name);
+    PERROR1 ("fopen (%s)", input_name);
 
   step = sequence[0];
   if (!(*step->file_recode) (step, input_file, output_file))
@@ -2051,7 +2042,7 @@ execute_pipe_sequence (const char *input_name, const char *output_name)
 
       if ((wait_status & ~(~0 << 8)) != 0
 	  && (wait_status & ~(~0 << 8)) != SIGPIPE)
-	error (0, 0, "Child process wait status is 0x%0.2x",
+	error (0, 0, _("Child process wait status is 0x%0.2x"),
 	       wait_status);
 
       /* Check for a non-zero exit from the terminating child.  */
@@ -2088,24 +2079,24 @@ execute_sequence (const char *input_name, const char *output_name)
   const STEP *step;		/* current step */
   int reversible;		/* reversibility of recoding */
 
-#ifdef MSDOS_or_OS2
+#if MSDOS_or_OS2
   if (!input_name)
     setmode (fileno (stdin), O_BINARY);
   if (!output_name)
     setmode (fileno (stdout), O_BINARY);
-#ifdef __EMX__
+# ifdef __EMX__
   {
     extern int _fmode_bin;
     _fmode_bin = 1;
   }
-#else
+# else
   _fmode = O_BINARY;
-#endif
+# endif
 #endif
 
   if (verbose_option && input_name)
     {
-      fprintf (stderr, "Recoding %s...", input_name);
+      fprintf (stderr, _("Recoding %s..."), input_name);
       fflush (stderr);
     }
 
@@ -2113,16 +2104,16 @@ execute_sequence (const char *input_name, const char *output_name)
     switch (sequence_strategy)
       {
       case STRATEGY_UNDECIDED:
-	error (EXIT_FAILURE, 0, "Internal error - strategy undecided");
+	error (EXIT_FAILURE, 0, _("Internal error - strategy undecided"));
 
       case SEQUENCE_WITH_PIPE:
-#ifdef HAVE_PIPE
+#if HAVE_PIPE
 	reversible = execute_pipe_sequence (input_name, output_name);
 	break;
 #endif
 
       case SEQUENCE_WITH_POPEN:
-#ifdef HAVE_POPEN
+#if HAVE_POPEN
 	reversible = execute_popen_sequence (input_name, output_name);
 	break;
 #endif
@@ -2139,14 +2130,12 @@ execute_sequence (const char *input_name, const char *output_name)
       if (!input_name)
 	input_file = stdin;
       else if (input_file = fopen (input_name, "r"), input_file == NULL)
-	error (EXIT_FAILURE, errno, "%s:%d: fopen (%s)",
-	       __FILE__, __LINE__, input_name);
+	PERROR1 ("fopen (%s)", input_name);
 
       if (!output_name)
 	output_file = stdout;
       else if (output_file = fopen (output_name, "w"), output_file == NULL)
-	error (EXIT_FAILURE, errno, "%s:%d: fopen (%s)",
-	       __FILE__, __LINE__, output_name);
+	PERROR1 ("fopen (%s)", output_name);
 
       if (length_of_sequence == 1)
 	{
@@ -2167,7 +2156,7 @@ execute_sequence (const char *input_name, const char *output_name)
 
   if (verbose_option && input_name)
     {
-      fprintf (stderr, " done\n");
+      fprintf (stderr, _(" done\n"));
       fflush (stderr);
     }
   return reversible;
@@ -2218,7 +2207,7 @@ perform_auto_check_one (CHARSET *before, CHARSET *after)
 	{
 	  print_truncated_charset_name (before->name);
 	  print_truncated_charset_name (after->name);
-	  printf ("  UNACHIEVABLE\n");
+	  printf (_("  UNACHIEVABLE\n"));
 	}
     }
   else
@@ -2242,17 +2231,17 @@ perform_auto_check_one (CHARSET *before, CHARSET *after)
       /* Check and report codes which should be aliases.  */
 
       if (length_of_sequence == 0)
-	quality_string = "ONE to SAME";
+	quality_string = _("ONE to SAME");
 
       /* Make the report.  */
 
       print_truncated_charset_name (before->name);
       print_truncated_charset_name (after->name);
       printf ("  %-16s", quality_string);
-      printf ("steps: %d", saved_length_of_sequence);
+      printf (_("steps: %d"), saved_length_of_sequence);
 
       if (length_of_sequence != saved_length_of_sequence)
-	printf (", %d saved by merging",
+	printf (_(", %d saved by merging"),
 		saved_length_of_sequence - length_of_sequence);
       printf ("\n");
 
@@ -2276,13 +2265,13 @@ perform_auto_check (CHARSET *charset)
   CHARSET *before;
   CHARSET *after;
 
-  if (charset->ignore)
-    error (EXIT_FAILURE, 0, "Cannot auto check the ignored charset");
-  if (charset == rfc1345)
-    error (EXIT_FAILURE, 0, "Cannot auto check on %s", charset->name);
-
   if (charset)
     {
+      if (charset->ignore)
+	error (EXIT_FAILURE, 0, _("Cannot auto check the ignored charset"));
+      if (charset == rfc1345)
+	error (EXIT_FAILURE, 0, _("Cannot auto check on %s"), charset->name);
+
       for (before = charset_array;
 	   before < charset_array + number_of_charsets;
 	   before++)
@@ -2321,15 +2310,15 @@ void
 usage (int status)
 {
   if (status != EXIT_SUCCESS)
-    fprintf (stderr, "Try `%s %s' for more information.\n", program_name,
+    fprintf (stderr, _("Try `%s %s' for more information.\n"), program_name,
 	     decoding_charset_flag ? "--list" : "--help");
   else
     {
-      printf ("GNU %s %s\n", PRODUCT, VERSION);
-      printf ("\
+      printf ("GNU %s %s\n", PACKAGE, VERSION);
+      printf (_("\
 \n\
-Usage: %s [OPTION]... [CHARSET]\n", program_name);
-      fputs ("\
+Usage: %s [OPTION]... [CHARSET]\n"), program_name);
+      fputs (_("\
 Mandatory or optional arguments to long options are mandatory or optional\n\
 for short options too.\n\
 \n\
@@ -2344,9 +2333,9 @@ FORMAT is a word among decimal, octal, hexadecimal or full (which may be\n\
 abbreviated to one of `dohf'), it defaults to just the canonical name.\n\
 With -k, possible before charsets are listed for the given after CHARSET,\n\
 both being RFC1345 charsets, with PAIRS of the form `BEF1:AFT1,BEF2:AFT2,...\n\
-and BEFs and AFTs being codes.  All codes are given as decimal numbers.\n",
+and BEFs and AFTs being codes.  All codes are given as decimal numbers.\n"),
 	      stdout);
-      fputs ("\
+      fputs (_("\
 Option -l with no FORMAT nor CHARSET list all charsets, also see the Texinfo\n\
 documentation.  My preferred charsets are (each user has preferences):\n\
 \n\
@@ -2354,12 +2343,12 @@ documentation.  My preferred charsets are (each user has preferences):\n\
   ibmpc      IBM-PC 8-bit characters, with proper newlines\n\
   latex      LaTeX coding of foreign and diacriticized characters\n\
   latin1     ISO Latin-1 8-bit extension of ASCII\n\
-  texte      Easy French convention for transmitting email messages\n",
+  texte      Easy French convention for transmitting email messages\n"),
 	     stdout);
-      printf ("\
+      printf (_("\
 \n\
-Usage: %s [OPTION]... [BEFORE]:[AFTER] [FILE]...\n", program_name);
-      fputs ("\
+Usage: %s [OPTION]... [BEFORE]:[AFTER] [FILE]...\n"), program_name);
+      fputs (_("\
 \n\
   -c, --colons            use colons instead of double quotes for diaeresis\n\
   -d, --diacritics        limit conversion to diacritics or alike for LaTeX\n\
@@ -2367,30 +2356,30 @@ Usage: %s [OPTION]... [BEFORE]:[AFTER] [FILE]...\n", program_name);
                           (BEWARE: in this version, -f is always selected)\n\
   -g, --graphics          approximate IBMPC rulers by ASCII graphics\n\
   -h, --header[=NAME]     write C code with table NAME on stdout, then exit\n\
-  -i, --sequence=files    use intermediate files for sequencing passes\n",
+  -i, --sequence=files    use intermediate files for sequencing passes\n"),
 	     stdout);
 
-#ifdef HAVE_POPEN
-      fputs ("\
-  -o, --sequence=popen    use popen machinery for sequencing passes\n",
+#if HAVE_POPEN
+      fputs (_("\
+  -o, --sequence=popen    use popen machinery for sequencing passes\n"),
 	     stdout);
 #else
-      fputs ("\
-  -o, --sequence=popen    same as -i (on this system)\n",
+      fputs (_("\
+  -o, --sequence=popen    same as -i (on this system)\n"),
 	     stdout);
 #endif
 
-#ifdef HAVE_PIPE
-      fputs ("\
-  -p, --sequence=pipe     use pipe machinery for sequencing passes\n",
+#if HAVE_PIPE
+      fputs (_("\
+  -p, --sequence=pipe     use pipe machinery for sequencing passes\n"),
 	     stdout);
 #else
-      fputs ("\
-  -p, --sequence=pipe     same as -o (on this system)\n",
+      fputs (_("\
+  -p, --sequence=pipe     same as -o (on this system)\n"),
 	     stdout);
 #endif
 
-      fputs ("\
+      fputs (_("\
   -q, --quiet, --silent   inhibit messages about irreversible recodings\n\
   -s, --strict            use strict mappings, even loose characters\n\
   -t, --touch             touch the recoded files after replacement\n\
@@ -2399,10 +2388,10 @@ Usage: %s [OPTION]... [BEFORE]:[AFTER] [FILE]...\n", program_name);
 \n\
 If none of -i, -o and -p are given, presume -p if no FILE, else -i.\n\
 Each FILE is recoded over itself, destroying the original.  If no\n\
-FILE is specified, then act as a filter and recode stdin to stdout.\n",
+FILE is specified, then act as a filter and recode stdin to stdout.\n"),
 	     stdout);
-      printf ("\
-BEFORE and AFTER both default to `%s' when needed.\n", DEFAULT_CHARSET);
+      printf (_("\
+BEFORE and AFTER both default to `%s' when needed.\n"), DEFAULT_CHARSET);
     }
   exit (status);
 }
@@ -2460,19 +2449,20 @@ main (int argc, char *const *argv)
   const char *input_name;	/* input file name */
   char output_name[200];	/* output file name */
   FILE *file;			/* file to check or stat */
-#ifdef MSDOS
+#if MSDOS
   struct ftime stamp_stat;	/* input file time stamps */
-#else /* not MSDOS */
+#else
   struct stat stamp_stat;	/* input file time stamps */
   time_t stamp_utime[2];	/* recoded file time stamps */
-#endif /* not MSDOS */
+#endif
   char *cursor;			/* all purpose cursor */
 
   /* Decode command options.  */
 
-  program_path = argv[0];
-  program_name = strrchr (program_path, '/');
-  program_name = program_name ? program_name + 1 : program_path;
+  program_name = argv[0];
+  setlocale (LC_ALL, "");
+
+  error_print_progname = error_print_prefix;
 
   if (argc == 1)
     usage (EXIT_SUCCESS);
@@ -2492,11 +2482,11 @@ main (int argc, char *const *argv)
 	switch (argmatch (optarg, sequence_strings))
 	  {
 	  case -2:
-	    error (0, 0, "Ambiguous sequence `%s'", optarg);
+	    error (0, 0, _("Ambiguous sequence `%s'"), optarg);
 	    usage (EXIT_FAILURE);
 
 	  case -1:
-	    error (0, 0, "Unknown sequence `%s'", optarg);
+	    error (0, 0, _("Unknown sequence `%s'"), optarg);
 	    usage (EXIT_FAILURE);
 
 	  case 0:
@@ -2518,7 +2508,21 @@ main (int argc, char *const *argv)
 	break;
 
       case 'C':
-	fprintf (stderr, "%s", copyright_string);
+	fprintf (stderr, "%s", _("\
+This program is free software; you can redistribute it and/or modify\n\
+it under the terms of the GNU General Public License as published by\n\
+the Free Software Foundation; either version 2, or (at your option)\n\
+any later version.\n\
+\n\
+This program is distributed in the hope that it will be useful,\n\
+but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
+GNU General Public License for more details.\n\
+\n\
+You should have received a copy of the GNU General Public License\n\
+along with this program; if not, write to the Free Software\n\
+Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n"));
+
 	exit (EXIT_SUCCESS);
 
       case 'c':
@@ -2557,11 +2561,11 @@ main (int argc, char *const *argv)
 	  switch (argmatch (optarg, format_strings))
 	    {
 	    case -2:
-	      error (0, 0, "Ambiguous format `%s'", optarg);
+	      error (0, 0, _("Ambiguous format `%s'"), optarg);
 	      usage (EXIT_FAILURE);
 
 	    case -1:
-	      error (0, 0, "Unknown format `%s'", optarg);
+	      error (0, 0, _("Unknown format `%s'"), optarg);
 	      usage (EXIT_FAILURE);
 
 	    case 0:
@@ -2613,7 +2617,7 @@ main (int argc, char *const *argv)
 
   if (ascii_graphics && strict_mapping)
     {
-      error (0, 0, "Currently, -s is ignored when -g is selected");
+      error (0, 0, _("Currently, -s is ignored when -g is selected"));
       strict_mapping = 0;
     }
 
@@ -2624,7 +2628,7 @@ main (int argc, char *const *argv)
 
   if (show_version)
     {
-      printf ("GNU %s %s\n", PRODUCT, VERSION);
+      printf ("GNU %s %s\n", PACKAGE, VERSION);
       exit (EXIT_SUCCESS);
     }
 
@@ -2700,7 +2704,7 @@ main (int argc, char *const *argv)
   length_of_sequence = 0;
   find_sequence (before_charset, after_charset);
   if (length_of_sequence < 0)
-    error (EXIT_FAILURE, 0, "No way to recode from %s to %s",
+    error (EXIT_FAILURE, 0, _("No way to recode from %s to %s"),
 	   before_charset->name, after_charset->name);
 
   simplify_sequence ();
@@ -2710,10 +2714,10 @@ main (int argc, char *const *argv)
   if (make_header_mode)
     {
       if (length_of_sequence == 0)
-	error (EXIT_FAILURE, 0, "Recoding is trivial, not worth a table");
+	error (EXIT_FAILURE, 0, _("Recoding is trivial, not worth a table"));
       if (length_of_sequence > 1
 	  || !(sequence[0]->one_to_one || sequence[0]->one_to_many))
-	error (EXIT_FAILURE, 0, "Recoding is too complex for a mere table");
+	error (EXIT_FAILURE, 0, _("Recoding is too complex for a mere table"));
 
       output_header_file ();
       exit (EXIT_SUCCESS);
@@ -2753,14 +2757,13 @@ main (int argc, char *const *argv)
 	    /* Check if the file can be read and rewritten.  */
 
 	    if (file = fopen (input_name, "r+"), file == NULL)
-	      error (EXIT_FAILURE, errno, "%s:%d: fopen (%s)",
-		     __FILE__, __LINE__, input_name);
+	      PERROR1 ("fopen (%s)", input_name);
 
 	    /* Save the input file time stamp.  */
 
 	    if (!touch_option)
 	      {
-#ifdef MSDOS
+#if MSDOS
 		getftime (fileno (file), &stamp_stat);
 #else
 		fstat (fileno (file), &stamp_stat);
@@ -2771,7 +2774,7 @@ main (int argc, char *const *argv)
 
 	    /* Choose an output file in the same directory.  */
 
-#ifdef MSDOS_or_OS2
+#if MSDOS_or_OS2
 
 	    strcpy (output_name, input_name);
 	    for (cursor = output_name + strlen (output_name);
@@ -2798,36 +2801,32 @@ main (int argc, char *const *argv)
 	      {
 		reversible = 0;
 		if (!quiet_mode)
-		  error (0, 0, "%s: Recoding is not reversible", input_name);
+		  error (0, 0, _("%s: Recoding is not reversible"),
+			 input_name);
 	      }
 
 	    /* Move the new file over the original.  */
 
 	    if (unlink (input_name) < 0)
-	      error (EXIT_FAILURE, errno, "%s:%d: unlink (%s)",
-		     __FILE__, __LINE__, input_name);
-#ifdef HAVE_RENAME
+	      PERROR1 ("unlink (%s)", input_name);
+#if HAVE_RENAME
 	    if (rename (output_name, input_name) < 0)
-	      error (EXIT_FAILURE, errno, "%s:%d: rename (%s, %s)",
-		     __FILE__, __LINE__, output_name, input_name);
+	      PERROR2 ("rename (%s, %s)", output_name, input_name);
 #else
 	    if (link (output_name, input_name) < 0)
-	      error (EXIT_FAILURE, errno, "%s:%d: link (%s, %s)",
-		     __FILE__, __LINE__, output_name, input_name);
+	      PERROR2 ("link (%s, %s)", output_name, input_name);
 	    if (unlink (output_name) < 0)
-	      error (EXIT_FAILURE, errno, "%s:%d: unlink (%s)",
-		     __FILE__, __LINE__, output_name);
+	      PERROR1 ("unlink (%s)", output_name);
 #endif
 
 	    /* Adjust the time stamp for the new file.  */
 
 	    if (!touch_option)
 	      {
-#ifdef MSDOS
+#if MSDOS
 		file = fopen (input_name, "r");
 		if (file == NULL)
-		  error (EXIT_FAILURE, errno, "%s:%d: fopen (%s)",
-			 __FILE__, __LINE__, input_name);
+		  PERROR1 ("fopen (%s)", input_name);
 		setftime (fileno (file), &stamp_stat);
 		fclose (file);
 #else
@@ -2851,7 +2850,7 @@ main (int argc, char *const *argv)
 	{
 	  reversible = 0;
 	  if (!quiet_mode)
-	    error (0, 0, "Recoding is not reversible");
+	    error (0, 0, _("Recoding is not reversible"));
 	}
     }
 
